@@ -101,10 +101,10 @@ as.character.mixed_symbolic_units = function(x, ...) {
 .cat_units_table <- function(x) {
   cat("Mixed units: ")
   if (!length(x)) return()
-
-  tbl = table(as.character(units(x)))
-  tbl = paste(names(tbl), " (", as.numeric(tbl), ")", sep = "")
-  cat(paste(tbl, collapse = ", "), "\n")
+  x <- as.character(units(x))
+  u <- unique(x)
+  n <- tabulate(factor(x, u))
+  cat(paste(u, " (", n, ")", sep = "", collapse = ", "), "\n")
 }
 
 #' @export
@@ -142,3 +142,11 @@ Ops.mixed_units = function(e1, e2) {
 		ret = .as.mixed_units(ret)
 	ret
 }
+
+#' @export
+unique.mixed_units <- function(x, incomparables = FALSE, ...) {
+  .as.mixed_units(NextMethod())
+}
+
+#' @export
+as.data.frame.mixed_units <- as.data.frame.AsIs
